<?php

namespace Teknisa\MAN\Factories;

use Doctrine\DBAL\Connection;

class RelVendaDiaHoraFactory {
    protected $connection;

    public function __construct(Connection $connection) {
        $this->connection = $connection;
    }

    public function findVendasPorHora($params) {
        $paramRelatorio = [
            'P_CDFILIAL'   => $params['P_CDFILIAL'],
            'P_TCDFILIAL'  => $params['P_TCDFILIAL'],
            'P_CDCAIXA'    => $params['P_CDCAIXA'],
            'P_TCDCAIXA'   => $params['P_TCDCAIXA'],
            'P_CDLOJA'     => $params['P_CDLOJA'],
            'P_TCDLOJA'    => $params['P_TCDLOJA'],
            'P_DTVENDAINI' => $params['P_DTVENDAINI'],
            'P_DTVENDAFIN' => $params['P_DTVENDAFIN'],
            'P_HRINI'      => $params['P_DTVENDAINI'].' '.$params['P_HRINI'],
            'P_HRFIN'      => $params['P_DTVENDAFIN'].' '.$params['P_HRFIN'],
            'P_IDIMPHR'    => $params['P_IDIMPHR'],
            'P_MINUTOS'    => $params['P_MINUTOS']
        ];
        
        $sql = "
            SELECT A.CDLOJA || ' - ' || A.NMLOJA AS LOJA, A.CDCAIXA || ' - ' || A.NMCAIXA AS CAIXA, A.DTENTRVENDA, 
                    SUM(TRUNC(B.QTPRODVEND * (NVL(B.VRUNITVEND, 0) + NVL(B.VRUNITVENDCL, 0)), 2) - NVL(B.VRDESITVEND, 0) + NVL(B.VRACRITVEND, 0)) AS VRTOTAL,
                    SUM(B.QTPRODVEND) AS QTDE, COUNT(DISTINCT A.NRSEQVENDA) AS QTDECUPONS
                FROM (
                    SELECT V.CDFILIAL, V.CDCAIXA, V.NRSEQVENDA, V.CDLOJA, V.DTVENDA,  C.NMCAIXA, L.NMLOJA,
                            TRUNC(V.DTVENDA - MOD(TO_NUMBER(TO_CHAR(V.DTVENDA,'MI')) + TO_NUMBER(TO_CHAR(V.DTVENDA, 'HH24')) * 60, :P_MINUTOS)/24/60, 'MI') AS DTENTRVENDA
                        FROM VENDA V, (
                            SELECT DISTINCT CDFILIAL, CDCAIXA, NMCAIXA
                                FROM CAIXA
                                WHERE ((INSTR(:P_CDFILIAL, CDFILIAL) <> 0) OR (:P_TCDFILIAL = 'T'))
                                    AND ((INSTR(:P_CDCAIXA, CDFILIAL||CDCAIXA) <> 0) OR (:P_TCDCAIXA = 'T'))
                        ) C,
                        (
                            SELECT DISTINCT CDFILIAL, CDLOJA, NMLOJA
                                    FROM LOJA
                                WHERE ((INSTR(:P_CDFILIAL, CDFILIAL) <> 0) OR (:P_TCDFILIAL = 'T'))
                                    AND ((INSTR(:P_CDLOJA, CDLOJA) <> 0) OR (:P_TCDLOJA = 'T'))
                        ) L
                        WHERE V.DTENTRVENDA BETWEEN :P_DTVENDAINI AND :P_DTVENDAFIN
                            AND ((V.DTVENDA BETWEEN TO_DATE(:P_HRINI || ':00', 'DD/MM/YYYY HH24:MI:SS') AND TO_DATE(:P_HRFIN || ':59', 'DD/MM/YYYY HH24:MI:SS')) OR (:P_IDIMPHR = 'N'))
                            AND V.IDSITUVENDA = 'O'
                            AND ((INSTR(:P_CDFILIAL, V.CDFILIAL) <> 0) OR (:P_TCDFILIAL = 'T'))
                            AND V.CDCAIXA = C.CDCAIXA
                            AND V.CDFILIAL = C.CDFILIAL
                            AND V.CDLOJA = L.CDLOJA
                            AND V.CDFILIAL = L.CDFILIAL
                ) A, ITEMVENDA B
                WHERE A.CDFILIAL = B.CDFILIAL
                    AND A.CDCAIXA = B.CDCAIXA
                    AND A.NRSEQVENDA = B.NRSEQVENDA
                GROUP BY A.CDLOJA, A.NMLOJA, A.CDCAIXA, A.NMCAIXA, A.DTENTRVENDA
            ORDER BY A.CDLOJA, A.CDCAIXA, A.DTENTRVENDA
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($paramRelatorio);
        
        $result = $std->fetchAll();

        for($i = 0; $i < count($result); $i++) {
            if($result[$i]['QTDECUPONS'] == 0) {
                $result[$i]['TICKETMEDIO']    = 0;
                $result[$i]['ITENSPORTICKET'] = 0;
            } else {
                $result[$i]['TICKETMEDIO']    = $result[$i]['VRTOTAL'] / $result[$i]['QTDECUPONS'];
                $result[$i]['ITENSPORTICKET'] = $result[$i]['QTDE']    / $result[$i]['QTDECUPONS'];
            }
        }

        return $result;
    }

    public function findVendasPorHoraDataVenda($params) {
        $paramRelatorio = [
            'P_CDFILIAL'   => $params['P_CDFILIAL'],
            'P_TCDFILIAL'  => $params['P_TCDFILIAL'],
            'P_CDCAIXA'    => $params['P_CDCAIXA'],
            'P_TCDCAIXA'   => $params['P_TCDCAIXA'],
            'P_CDLOJA'     => $params['P_CDLOJA'],
            'P_TCDLOJA'    => $params['P_TCDLOJA'],
            'P_DTVENDAINI' => $params['P_DTVENDAINI'],
            'P_DTVENDAFIN' => $params['P_DTVENDAFIN'],
            'P_HRINI'      => $params['P_HRINI'],
            'P_HRFIN'      => $params['P_HRFIN'],
            'P_MINUTOS'    => $params['P_MINUTOS']
        ];
        
        $sql = "
            SELECT A.CDLOJA || ' - ' || A.NMLOJA AS LOJA, A.CDCAIXA || ' - ' || A.NMCAIXA AS CAIXA, A.DTVENDA,
                    SUM(TRUNC(B.QTPRODVEND * (NVL(B.VRUNITVEND, 0) + NVL(B.VRUNITVENDCL, 0)), 2) - NVL(B.VRDESITVEND, 0) +  NVL(B.VRACRITVEND, 0)) AS VRTOTAL,
                    SUM(B.QTPRODVEND) AS QTDE, COUNT(DISTINCT A.NRSEQVENDA) AS QTDECUPONS 
                FROM (
                    SELECT DISTINCT LC.CDFILIAL, LC.CDCAIXA, V.NRSEQVENDA, LC.CDLOJA, V.DTENTRVENDA, LC.NMCAIXA, LC.NMLOJA,
                            TRUNC(DTVENDA - MOD(TO_NUMBER(TO_CHAR(DTVENDA, 'MI')) + TO_NUMBER(TO_CHAR(DTVENDA, 'HH24')) * 60, :P_MINUTOS)/24/60, 'MI') AS DTVENDA
                        FROM VENDA V, (
                            SELECT DISTINCT C.CDCAIXA, C.NMCAIXA, L.CDLOJA, L.CDFILIAL, L.NMLOJA
                                FROM CAIXA C, (
                                    SELECT DISTINCT CDFILIAL, CDLOJA, NMLOJA
                                        FROM LOJA
                                        WHERE ((INSTR(:P_CDFILIAL, CDFILIAL) <> 0) OR (:P_TCDFILIAL = 'T'))
                                            AND ((INSTR(:P_CDLOJA, CDFILIAL||CDLOJA) <> 0) OR (:P_TCDLOJA = 'T'))
                                ) L
                                WHERE C.CDFILIAL = L.CDFILIAL
                                    AND C.CDLOJA = L.CDLOJA
                                    AND ((INSTR(:P_CDCAIXA, L.CDFILIAL||CDCAIXA) <> 0) OR (:P_TCDCAIXA = 'T'))
                        ) LC
                        WHERE TRUNC(DTVENDA) BETWEEN :P_DTVENDAINI AND :P_DTVENDAFIN
                            AND TO_DATE(TO_CHAR(DTVENDA, 'HH24MI'), 'HH24MI') BETWEEN TO_DATE(:P_HRINI, 'HH24MI') AND TO_DATE(:P_HRFIN, 'HH24MI')
                            AND IDSITUVENDA = 'O'
                            AND ((INSTR(:P_CDFILIAL, V.CDFILIAL) <> 0) OR (:P_TCDFILIAL = 'T'))
                            AND V.CDFILIAL = LC.CDFILIAL
                            AND V.CDLOJA = LC.CDLOJA
                            AND V.CDCAIXA = LC.CDCAIXA
                ) A, ITEMVENDA B
                WHERE A.CDFILIAL = B.CDFILIAL
                    AND A.CDCAIXA = B.CDCAIXA
                    AND A.NRSEQVENDA = B.NRSEQVENDA
                GROUP BY A.CDLOJA, A.NMLOJA, A.CDCAIXA, A.NMCAIXA, A.DTVENDA
            ORDER BY A.CDLOJA, A.CDCAIXA, A.DTVENDA
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($paramRelatorio);
        
        $result = $std->fetchAll();
        
        for($i = 0; $i < count($result); $i++) {
            if($result[$i]['QTDECUPONS'] == 0) {
                $result[$i]['TICKETMEDIO']    = 0;
                $result[$i]['ITENSPORTICKET'] = 0;
            } else {
                $result[$i]['TICKETMEDIO']    = $result[$i]['VRTOTAL'] / $result[$i]['QTDECUPONS'];
                $result[$i]['ITENSPORTICKET'] = $result[$i]['QTDE']    / $result[$i]['QTDECUPONS'];
            }
        }
        
        return $result;
    }

    public function findVendasPorHoraProduto($params) {
        $paramRelatorio = [
            'P_CDFILIAL'   => $params['P_CDFILIAL'],
            'P_TCDFILIAL'  => $params['P_TCDFILIAL'],
            'P_CDCAIXA'    => $params['P_CDCAIXA'],
            'P_TCDCAIXA'   => $params['P_TCDCAIXA'],
            'P_CDLOJA'     => $params['P_CDLOJA'],
            'P_TCDLOJA'    => $params['P_TCDLOJA'],
            'P_DTVENDAINI' => $params['P_DTVENDAINI'],
            'P_DTVENDAFIN' => $params['P_DTVENDAFIN'],
            'P_HRINI'      => $params['P_HRINI'],
            'P_HRFIN'      => $params['P_HRFIN'],
            'P_MINUTOS'    => $params['P_MINUTOS'],
            'P_CDPRODLIST' => $params['P_CDPRODLIST'],
            'P_CDARVPRODI' => $params['P_CDARVPRODI'],
            'P_CDARVPRODF' => $params['P_CDARVPRODF'],
            'P_TCDPRODUTO' => $params['P_TCDPRODUTO']
        ];

        $sql = "
            SELECT A.CDLOJA || ' - ' || C.NMLOJA AS LOJA, A.CDCAIXA || ' - ' || D.NMCAIXA AS CAIXA,
                    P.CDARVPROD, P.NMPRODUTO, A.DTENTRVENDA,
                    SUM(TRUNC(B.QTPRODVEND * (NVL(B.VRUNITVEND, 0) + NVL(B.VRUNITVENDCL, 0)), 2) - NVL(B.VRDESITVEND, 0) + NVL(B.VRACRITVEND, 0)) AS VRTOTAL,
                    SUM(B.QTPRODVEND) AS QTDE, COUNT(DISTINCT A.CDFILIAL||A.CDCAIXA||A.NRSEQVENDA) AS QTDECUPONS
                FROM (
                    SELECT CDFILIAL, CDCAIXA, NRSEQVENDA, CDLOJA, DTVENDA,
                            TRUNC(DTVENDA - MOD(TO_NUMBER(TO_CHAR(DTVENDA, 'MI')) + TO_NUMBER(TO_CHAR(DTVENDA, 'HH24')) * 60, :P_MINUTOS)/24/60, 'MI') AS DTENTRVENDA
                        FROM VENDA
                        WHERE DTENTRVENDA BETWEEN :P_DTVENDAINI AND :P_DTVENDAFIN
                            AND TO_DATE(TO_CHAR(DTVENDA, 'HH24MI'), 'HH24MI') BETWEEN TO_DATE(:P_HRINI, 'HH24MI') AND TO_DATE(:P_HRFIN, 'HH24MI')
                            AND IDSITUVENDA = 'O'
                            AND ((INSTR(:P_CDFILIAL, CDFILIAL) <> 0) OR (:P_TCDFILIAL = 'T'))
                            AND ((INSTR(:P_CDLOJA, CDLOJA)     <> 0) OR (:P_TCDLOJA   = 'T'))
                            AND ((INSTR(:P_CDCAIXA, CDFILIAL||CDCAIXA) <> 0) OR (:P_TCDCAIXA  = 'T'))
                ) A, ITEMVENDA B, LOJA C, CAIXA D, PRODUTO P
                WHERE A.CDFILIAL = B.CDFILIAL
                    AND A.CDCAIXA    = B.CDCAIXA
                    AND A.NRSEQVENDA = B.NRSEQVENDA
                    AND A.CDFILIAL   = C.CDFILIAL
                    AND A.CDLOJA     = C.CDLOJA
                    AND A.CDFILIAL   = D.CDFILIAL
                    AND A.CDCAIXA    = D.CDCAIXA
                    AND B.CDPRODUTO  = P.CDPRODUTO  
                    AND (((((INSTR(:P_CDPRODLIST, P.CDPRODUTO) > 0) OR (P.CDARVPROD BETWEEN :P_CDARVPRODI AND :P_CDARVPRODF)))) OR (:P_TCDPRODUTO = 'T'))
                GROUP BY A.CDLOJA, C.NMLOJA, A.CDCAIXA, D.NMCAIXA, A.DTENTRVENDA, P.CDARVPROD, P.NMPRODUTO
            ORDER BY A.CDLOJA, A.CDCAIXA, A.DTENTRVENDA, P.CDARVPROD, P.NMPRODUTO
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($paramRelatorio);
        
        $result = $std->fetchAll();
        
        for($i = 0; $i < count($result); $i++) {
            if($result[$i]['QTDECUPONS'] == 0) {
                $result[$i]['TICKETMEDIO']    = 0;
                $result[$i]['ITENSPORTICKET'] = 0;
            } else {
                $result[$i]['TICKETMEDIO']    = $result[$i]['VRTOTAL'] / $result[$i]['QTDECUPONS'];
                $result[$i]['ITENSPORTICKET'] = $result[$i]['QTDE']    / $result[$i]['QTDECUPONS'];
            }
            
            $codigoProduto = $this->mascaraPersonalizada($result[$i]['CDARVPROD']);
            
            $result[$i]['PRODUTO'] = $codigoProduto.' - '.$result[$i]['NMPRODUTO'];
        }
        
        return $result;
    }

    public function findVenasPorHoraProdutoConsolidadoLoja($params) {
        $paramRelatorio = [
            'P_CDFILIAL'   => $params['P_CDFILIAL'],
            'P_TCDFILIAL'  => $params['P_TCDFILIAL'],
            'P_CDCAIXA'    => $params['P_CDCAIXA'],
            'P_TCDCAIXA'   => $params['P_TCDCAIXA'],
            'P_CDLOJA'     => $params['P_CDLOJA'],
            'P_TCDLOJA'    => $params['P_TCDLOJA'],
            'P_DTVENDAINI' => $params['P_DTVENDAINI'],
            'P_DTVENDAFIN' => $params['P_DTVENDAFIN'],
            'P_HRINI'      => $params['P_HRINI'],
            'P_HRFIN'      => $params['P_HRFIN'],
            'P_MINUTOS'    => $params['P_MINUTOS'],
            'P_CDPRODLIST' => $params['P_CDPRODLIST'],
            'P_CDARVPRODI' => $params['P_CDARVPRODI'],
            'P_CDARVPRODF' => $params['P_CDARVPRODF'],
            'P_TCDPRODUTO' => $params['P_TCDPRODUTO']
        ];

        $sql = "
            SELECT FINAL.*, FINAL.CDLOJA || ' - ' || FINAL.NMLOJA AS LOJA, FINAL.NMPRODUTO,
                    DECODE(QTCUPOMPRODUTO, 0, 0, (VRPRODUTO / QTCUPOMPRODUTO)) AS TICKETMEDIO,
                    DECODE(QTCUPOMPRODUTO, 0, 0, (QTPRODUTO / QTCUPOMPRODUTO)) AS ITENSPORTICKET
                FROM (
                    SELECT DISTINCT REL.CDFILIAL, REL.CDLOJA, REL.NMLOJA, REL.CDARVPROD, REL.NMPRODUTO, REL.DTENTRVENDA,
                            COUNT(DISTINCT  REL.CDFILIAL||REL.CDLOJA||REL.CDARVPROD||TO_CHAR(REL.DTENTRVENDA, 'DD/MM/YYYY HH24:MI')||REL.NRSEQVENDA) OVER
                                (PARTITION BY REL.CDFILIAL, REL.CDLOJA, REL.CDARVPROD, TO_CHAR(REL.DTENTRVENDA, 'DD/MM/YYYY HH24:MI')) QTCUPOMPRODUTO,
                            SUM(QTPRODUTO) OVER
                                (PARTITION BY REL.CDFILIAL||REL.CDLOJA||REL.CDARVPROD||TO_CHAR(REL.DTENTRVENDA, 'DD/MM/YYYY HH24:MI')) QTPRODUTO,
                            SUM(VRPRODUTO) OVER
                                (PARTITION BY REL.CDFILIAL||REL.CDLOJA||REL.CDARVPROD||TO_CHAR(REL.DTENTRVENDA, 'DD/MM/YYYY HH24:MI')) VRPRODUTO,
                            COUNT(DISTINCT  REL.CDFILIAL||REL.CDLOJA||REL.NRSEQVENDA) OVER
                                (PARTITION BY REL.CDFILIAL, REL.CDLOJA) QTCUPOMLOJA,
                            COUNT(DISTINCT  REL.CDFILIAL||REL.CDLOJA||REL.NRSEQVENDA) OVER
                                (PARTITION BY REL.CDFILIAL) QTCUPOMTOTAL
                        FROM (
                            SELECT VE.CDFILIAL, VE.CDLOJA, LC.NMLOJA, PR.CDARVPROD, PR.NMPRODUTO, VE.NRSEQVENDA, VE.DTVENDA,
                                    TRUNC(DTVENDA - MOD(TO_NUMBER(TO_CHAR(DTVENDA, 'MI')) + TO_NUMBER(TO_CHAR(DTVENDA, 'HH24')) * 60, :P_MINUTOS)/24/60, 'MI') AS DTENTRVENDA,
                                    TRUNC(IT.QTPRODVEND * (NVL(IT.VRUNITVEND, 0) + NVL(IT.VRUNITVENDCL, 0)), 2) - NVL(IT.VRDESITVEND, 0) + NVL(IT.VRACRITVEND, 0) AS VRPRODUTO,
                                    IT.QTPRODVEND AS QTPRODUTO
                                FROM PRODUTO PR, VENDA VE, ITEMVENDA IT, (
                                    SELECT DISTINCT C.CDCAIXA, C.NMCAIXA, L.CDFILIAL, L.CDLOJA, L.NMLOJA
                                        FROM CAIXA C, (
                                            SELECT DISTINCT CDFILIAL, CDLOJA, NMLOJA
                                                FROM LOJA
                                                WHERE ((INSTR(:P_CDFILIAL, CDFILIAL) <> 0) OR (:P_TCDFILIAL = 'T'))
                                                    AND ((INSTR(:P_CDLOJA, CDFILIAL||CDLOJA) <> 0) OR (:P_TCDLOJA = 'T'))
                                        ) L
                                        WHERE C.CDFILIAL = L.CDFILIAL
                                            AND C.CDLOJA = L.CDLOJA
                                            AND ((INSTR(:P_CDCAIXA, L.CDFILIAL||C.CDCAIXA) <> 0) OR (:P_TCDCAIXA = 'T'))
                                ) LC
                                WHERE VE.CDFILIAL = IT.CDFILIAL
                                    AND VE.CDCAIXA  = IT.CDCAIXA
                                    AND VE.NRSEQVENDA = IT.NRSEQVENDA
                                    AND IT.CDPRODUTO  = PR.CDPRODUTO
                                    AND VE.DTENTRVENDA BETWEEN :P_DTVENDAINI AND :P_DTVENDAFIN
                                    AND TO_DATE(TO_CHAR(DTVENDA, 'HH24MI'), 'HH24MI') BETWEEN TO_DATE(:P_HRINI, 'HH24MI') AND TO_DATE(:P_HRFIN, 'HH24MI')
                                    AND VE.IDSITUVENDA = 'O'
                                    AND ((INSTR(:P_CDFILIAL, VE.CDFILIAL) <> 0) OR (:P_TCDFILIAL = 'T'))
                                    AND (((((INSTR(:P_CDPRODLIST, PR.CDPRODUTO) > 0) OR (PR.CDARVPROD BETWEEN :P_CDARVPRODI AND :P_CDARVPRODF)))) OR (:P_TCDPRODUTO = 'T'))
                                    AND VE.CDLOJA = LC.CDLOJA
                                    AND VE.CDCAIXA = LC.CDCAIXA
                                    AND VE.CDFILIAL = LC.CDFILIAL
                        ) REL
                ) FINAL
            ORDER BY CDFILIAL, CDLOJA, CDARVPROD, DTENTRVENDA
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($paramRelatorio);
        
        $result = $std->fetchAll();

        // aplicando mascara no produto
        for($i = 0; $i < count($result); $i++) {
            $codigoProduto = $this->mascaraPersonalizada($result[$i]['CDARVPROD']);
            
            $result[$i]['PRODUTO'] = $codigoProduto.' - '.$result[$i]['NMPRODUTO'];
        }
        
        return $result;
    }

    public function findVendasPorHoraConsolidadoFilial($params) {
        $paramRelatorio = [
            'P_CDFILIAL'   => $params['P_CDFILIAL'],
            'P_TCDFILIAL'  => $params['P_TCDFILIAL'],
            'P_CDLOJA'     => $params['P_CDLOJA'],
            'P_TCDLOJA'    => $params['P_TCDLOJA'],
            'P_DTVENDAINI' => $params['P_DTVENDAINI'],
            'P_DTVENDAFIN' => $params['P_DTVENDAFIN'],
            'P_HRINI'      => $params['P_HRINI'],
            'P_HRFIN'      => $params['P_HRFIN']
        ];

        $sql = "
            SELECT A.CDFILIAL || ' - ' || F.NMFILIAL AS FILIAL, A.DTENTRVENDA,
                    SUM(TRUNC(B.QTPRODVEND * (NVL(B.VRUNITVEND, 0) + NVL(B.VRUNITVENDCL, 0)), 2) - NVL(B.VRDESITVEND, 0) + NVL(B.VRACRITVEND, 0)) AS VRTOTAL,
                    SUM(B.QTPRODVEND) AS QTDE,
                    COUNT(DISTINCT A.NRSEQVENDA) AS QTDECUPONS 
                FROM (
                    SELECT CDFILIAL, CDCAIXA, NRSEQVENDA, CDLOJA, DTVENDA, DTENTRVENDA
                        FROM VENDA
                        WHERE ((INSTR(:P_CDFILIAL, CDFILIAL) <> 0) OR (:P_TCDFILIAL = 'T'))
                            AND IDSITUVENDA = 'O'
                            AND DTENTRVENDA BETWEEN :P_DTVENDAINI AND :P_DTVENDAFIN
                            AND ((INSTR(:P_CDLOJA, CDLOJA) <> 0) OR (:P_TCDLOJA = 'T'))
                            AND TO_DATE(TO_CHAR(DTVENDA, 'HH24MI'), 'HH24MI') BETWEEN TO_DATE(:P_HRINI, 'HH24MI') AND TO_DATE(:P_HRFIN, 'HH24MI')
                ) A, ITEMVENDA B, FILIAL F
                WHERE A.CDFILIAL = B.CDFILIAL
                    AND A.CDCAIXA    = B.CDCAIXA
                    AND A.NRSEQVENDA = B.NRSEQVENDA
                    AND A.CDFILIAL   = F.CDFILIAL
                GROUP BY A.CDFILIAL, F.NMFILIAL, A.DTENTRVENDA
            ORDER BY A.DTENTRVENDA
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($paramRelatorio);
        
        $result = $std->fetchAll();

        for($i = 0; $i < count($result); $i++) {
            if($result[$i]['QTDECUPONS'] == 0) {
                $result[$i]['TICKETMEDIO']    = 0;
                $result[$i]['ITENSPORTICKET'] = 0;
            } else {
                $result[$i]['TICKETMEDIO']    = $result[$i]['VRTOTAL'] / $result[$i]['QTDECUPONS'];
                $result[$i]['ITENSPORTICKET'] = $result[$i]['QTDE']    / $result[$i]['QTDECUPONS'];
            }
            
            $result[$i]['DTENTRVENDA'] = str_replace('00:00:00', '', $result[$i]['DTENTRVENDA']);
        }

        return $result;
    }

    public function findVendasPorHoraConsolidadoPeriodo($params) {
        $result = [];
        $paramRelatorio = [
            'P_CDFILIAL'   => $params['P_CDFILIAL'],
            'P_TCDFILIAL'  => $params['P_TCDFILIAL'],
            'P_CDLOJA'     => $params['P_CDLOJA'],
            'P_TCDLOJA'    => $params['P_TCDLOJA'],
            'P_DTVENDAINI' => $params['P_DTVENDAINI'],
            'P_DTVENDAFIN' => $params['P_DTVENDAFIN'],
            'P_HRINI'      => $params['P_HRINI'],
            'P_HRFIN'      => $params['P_HRFIN']
        ];

        $sql = "
            SELECT A.CDFILIAL, A.CDLOJA,
                    A.CDFILIAL || ' - ' || F.NMFILIAL AS FILIAL,
                    A.CDLOJA || ' - ' || C.NMLOJA AS LOJA,
                    TO_CHAR(A.DTVENDA, 'HH24') AS HORA,
                    SUM(TRUNC(B.QTPRODVEND * (NVL(B.VRUNITVEND, 0) + NVL(B.VRUNITVENDCL, 0)), 2) - NVL(B.VRDESITVEND, 0) + NVL(B.VRACRITVEND, 0)) AS VRTOTAL,
                    SUM(B.QTPRODVEND) AS QTDE,
                    COUNT(DISTINCT A.NRSEQVENDA) AS QTDECUPONS 
                FROM (
                    SELECT CDFILIAL, CDCAIXA, NRSEQVENDA, CDLOJA, DTVENDA
                        FROM VENDA
                        WHERE ((INSTR(:P_CDFILIAL, CDFILIAL) <> 0) OR (:P_TCDFILIAL = 'T'))
                            AND IDSITUVENDA = 'O' 
                            AND DTENTRVENDA BETWEEN :P_DTVENDAINI AND :P_DTVENDAFIN
                            AND TO_DATE(TO_CHAR(DTVENDA, 'HH24MI'), 'HH24MI') BETWEEN TO_DATE(:P_HRINI, 'HH24MI') AND TO_DATE(:P_HRFIN, 'HH24MI')
                ) A, ITEMVENDA B, LOJA C, FILIAL F
                WHERE A.CDFILIAL = B.CDFILIAL
                    AND A.CDCAIXA    = B.CDCAIXA
                    AND A.NRSEQVENDA = B.NRSEQVENDA
                    AND A.CDFILIAL   = C.CDFILIAL
                    AND A.CDLOJA     = C.CDLOJA
                    AND A.CDFILIAL   = F.CDFILIAL
                    AND ((INSTR(:P_CDLOJA, C.CDLOJA) <> 0) OR (:P_TCDLOJA = 'T'))
                GROUP BY A.CDLOJA, A.CDFILIAL, F.NMFILIAL, C.NMLOJA, TO_CHAR(A.DTVENDA, 'HH24')
            ORDER BY TO_CHAR(A.DTVENDA, 'HH24')
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($paramRelatorio);
        
        $result = $std->fetchAll();
        
        // pegando a porcentagem total
        $qryPorcentagem = "
            SELECT SUM(TRUNC(I.QTPRODVEND * (NVL(I.VRUNITVEND, 0) + NVL(I.VRUNITVENDCL, 0)), 2) - NVL(I.VRDESITVEND, 0) + NVL(I.VRACRITVEND, 0)) AS VRTOTALPORCENT
                FROM VENDA V, ITEMVENDA I
                WHERE DTENTRVENDA BETWEEN :P_DTVENDAINI AND :P_DTVENDAFIN
                    AND TO_DATE(TO_CHAR(DTVENDA, 'HH24MI'), 'HH24MI') BETWEEN TO_DATE(:P_HRINI, 'HH24MI') AND TO_DATE(:P_HRFIN, 'HH24MI')
                    AND V.IDSITUVENDA = 'O'
                    AND V.CDFILIAL    = :P_CDFILIAL
                    AND V.CDLOJA      = :P_CDLOJA    
                    AND V.CDFILIAL    = I.CDFILIAL
                    AND V.CDCAIXA     = I.CDCAIXA
                    AND V.NRSEQVENDA  = I.NRSEQVENDA
        ";

        for($i = 0; $i < count($result); $i++) {
            $paramPerc = [
                'P_DTVENDAINI' => $params['P_DTVENDAINI'],
                'P_DTVENDAFIN' => $params['P_DTVENDAFIN'],
                'P_CDFILIAL'   => $result[$i]['CDFILIAL'],
                'P_CDLOJA'     => $result[$i]['CDLOJA'],
                'P_HRINI'      => $params['P_HRINI'],
                'P_HRFIN'      => $params['P_HRFIN']
            ];

            $std = $this->connection->prepare($qryPorcentagem);
            $std->execute($paramPerc);
            
            $resultPerc = $std->fetchAll();
            
            if($result[$i]['QTDECUPONS'] == 0) {
                $result[$i]['TICKETMEDIO']    = 0;
                $result[$i]['ITENSPORTICKET'] = 0;
            } else {
                $result[$i]['TICKETMEDIO']    = $result[$i]['VRTOTAL'] / $result[$i]['QTDECUPONS'];
                $result[$i]['ITENSPORTICKET'] = $result[$i]['QTDE']    / $result[$i]['QTDECUPONS'];
            }

            $result[$i]['PORCTOTAL'] = ($result[$i]['VRTOTAL'] * 100) / $resultPerc[0]['VRTOTALPORCENT'];
        }

        return $result;
    }

    public function findVendasPorHoraModalidadeConsolidado($params) {
        $paramRelatorio = [
            'P_DTVENDAINI' => $params['P_DTVENDAINI'],
            'P_DTVENDAFIN' => $params['P_DTVENDAFIN'],
            'P_MINUTOS'    => $params['P_MINUTOS'],
            'P_CDLOJA'     => $params['P_CDLOJA'],
            'P_TCDLOJA'    => $params['P_TCDLOJA'],
            'P_CDFILIAL'   => $params['P_CDFILIAL'],
            'P_TCDFILIAL'  => $params['P_TCDFILIAL'],
            'P_HRINI'      => $params['P_HRINI'],
            'P_HRFIN'      => $params['P_HRFIN'],
            'P_IDIMPHR'    => $params['P_IDIMPHR']
        ];
        
        $sql = "
            SELECT V.CDFILIAL || ' - ' || V.NMFILIAL AS FILIAL,
                    V.CDLOJA || ' - ' || V.NMLOJA AS LOJA,
                    DECODE(P.IDIMPHR, 'S', V.DTVENDA, V.DTENTRVENDA) AS DATAVENDA, 
                    V.DTENTRVENDA, V.IDMOD AS MODALIDADE, 
                    :P_DTVENDAINI AS DATAINI, :P_DTVENDAFIN AS DATAFIN,
                    SUM(TRUNC(IT.QTPRODVEND * (NVL(IT.VRUNITVEND, 0) + NVL(IT.VRUNITVENDCL, 0)), 2) - NVL(IT.VRDESITVEND, 0) + NVL(IT.VRACRITVEND, 0)) AS VRTOTAL,
                    SUM(IT.QTPRODVEND) AS QTDE,
                    COUNT(DISTINCT V.CDCAIXA||V.NRSEQVENDA) AS QTDECUPONS
                FROM (
                    SELECT V.CDFILIAL, FLC.NMFILIAL, V.CDCAIXA, V.NRSEQVENDA, FLC.CDLOJA, 
                            V.DTENTRVENDA, FLC.NMLOJA, O.DSORIGEMVENDA AS IDMOD,
                            TO_CHAR(TRUNC(DTVENDA - MOD(TO_NUMBER(TO_CHAR(DTVENDA,'MI')) + TO_NUMBER(TO_CHAR(DTVENDA, 'HH24')) * 60, :P_MINUTOS)/24/60, 'MI'), 'DD/MM/YYYY HH24:MI')AS DTVENDA
                        FROM VENDA V, (
                            SELECT CDFILIAL, NMFILIAL, CDLOJA, NMLOJA, CDCAIXA
                                FROM V_FILIALLOJACAIXA
                                WHERE ((INSTR(:P_CDLOJA, CDLOJA) <> 0) OR (:P_TCDLOJA = 'T'))
                                    AND ((INSTR(:P_CDFILIAL, CDFILIAL) <> 0) OR (:P_TCDFILIAL = 'T'))
                        ) FLC, ORIGEMVENDAFOS O
                        WHERE DTENTRVENDA BETWEEN :P_DTVENDAINI AND :P_DTVENDAFIN
                            AND TO_DATE(TO_CHAR(DTVENDA, 'HH24MI'), 'HH24MI') BETWEEN TO_DATE(:P_HRINI, 'HH24MI') AND TO_DATE(:P_HRFIN, 'HH24MI')
                            AND IDSITUVENDA = 'O'
                            AND V.CDLOJA = FLC.CDLOJA
                            AND V.CDFILIAL = FLC.CDFILIAL
                            AND V.CDCAIXA = FLC.CDCAIXA
                            AND O.IDORIGEMVENDA = V.IDORIGEMVENDA
                ) V,
                (
                    SELECT :P_IDIMPHR AS IDIMPHR
                        FROM DUAL
                ) P, ITEMVENDA IT
                WHERE V.CDFILIAL = IT.CDFILIAL
                    AND V.CDCAIXA = IT.CDCAIXA
                    AND V.NRSEQVENDA = IT.NRSEQVENDA
                GROUP BY V.CDFILIAL, V.NMFILIAL, V.CDLOJA, V.NMLOJA, 
                        DECODE(P.IDIMPHR, 'S', V.DTVENDA, V.DTENTRVENDA),
                        V.DTENTRVENDA, V.IDMOD
            ORDER BY V.CDFILIAL, V.CDLOJA, V.NMLOJA, DECODE(P.IDIMPHR, 'S', V.DTVENDA, V.DTENTRVENDA), V.IDMOD
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($paramRelatorio);
        
        $result = $std->fetchAll();

        for($i = 0; $i < count($result); $i++) {
            if($result[$i]['QTDECUPONS'] == 0) {
                $result[$i]['TICKETMEDIO']    = 0;
                $result[$i]['ITENSPORTICKET'] = 0;
            } else {
                $result[$i]['TICKETMEDIO']    = $result[$i]['VRTOTAL'] / $result[$i]['QTDECUPONS'];
                $result[$i]['ITENSPORTICKET'] = $result[$i]['QTDE']    / $result[$i]['QTDECUPONS'];
            }
        }

        return $result;
    }

    public function findVendasPorHoraModalidadeProdutoConsolidado($params) {
        $paramRelatorio = [
            'DATAINI'    => $params['P_DTVENDAINI'],
            'DATAFIN'    => $params['P_DTVENDAFIN'],
            'MINUTOS'    => $params['P_MINUTOS'],
            'CDLOJA'     => $params['P_CDLOJA'],
            'TCDLOJA'    => $params['P_TCDLOJA'],
            'CDFILIAL'   => $params['P_CDFILIAL'],
            'CDOPERADOR' => $params['P_CDOPERADOR'],
            'HRINI'      => $params['P_HRINI'],
            'HRFIN'      => $params['P_HRFIN'],
            'IDIMPHR'    => $params['P_IDIMPHR']
        ];
        
        $sql = "
            SELECT V.CDFILIAL || ' - ' || V.NMFILIAL AS FILIAL, V.CDLOJA || ' - ' || V.NMLOJA AS LOJA, 
                    V.CDFILIAL, V.NMFILIAL, V.CDLOJA, V.NMLOJA, 
                    DECODE(D.IDIMPHR, 'S', V.DTVENDA, TO_CHAR(V.DTENTRVENDA, 'DD/MM/YYYY HH24:MI')) AS DATAVENDA, 
                    V.DTENTRVENDA, V.IDMOD, P.CDARVPROD, P.NMPRODUTO, :DATAINI AS DATAINI, :DATAFIN AS DATAFIN, 
                    SUM(TRUNC(IT.QTPRODVEND * (NVL(IT.VRUNITVEND, 0) + NVL(IT.VRUNITVENDCL, 0)), 2) - NVL(IT.VRDESITVEND, 0) + NVL(IT.VRACRITVEND, 0)) AS VRVENDAS, 
                    SUM(IT.QTPRODVEND) AS QTDE, COUNT(DISTINCT V.CDCAIXA || V.NRSEQVENDA) AS NRCUPONS,
                    SUM(TRUNC(IT.QTPRODVEND * (NVL(IT.VRUNITVEND, 0) + NVL(IT.VRUNITVENDCL, 0)), 2) - NVL(IT.VRDESITVEND, 0) + NVL(IT.VRACRITVEND, 0)) / COUNT(DISTINCT V.CDCAIXA || V.NRSEQVENDA) AS TICKETMEDIO, 
                    SUM(IT.QTPRODVEND) / COUNT(DISTINCT V.CDCAIXA || V.NRSEQVENDA) AS ITENSTICKET 
                FROM (
                    SELECT V.CDFILIAL, FLC.NMFILIAL, V.CDCAIXA, V.NRSEQVENDA, FLC.CDLOJA, V.DTENTRVENDA, 
                            FLC.NMLOJA, O.DSORIGEMVENDA AS IDMOD, 
                            TO_CHAR(TRUNC(DTVENDA - MOD(TO_NUMBER(TO_CHAR(DTVENDA, 'MI')) + TO_NUMBER(TO_CHAR(DTVENDA, 'HH24')) * 60, :MINUTOS)/24/60, 'MI'), 'DD/MM/YYYY HH24:MI') AS DTVENDA 
                        FROM VENDA V, (
                            SELECT CDFILIAL, NMFILIAL, CDLOJA, NMLOJA, CDCAIXA 
                                FROM V_FILIALLOJACAIXA 
                                WHERE ((INSTR(:CDLOJA, CDFILIAL || CDLOJA) <> 0) OR (:TCDLOJA = 'T')) 
                                    AND ((INSTR(:CDFILIAL, CDFILIAL) <> 0) OR CDFILIAL IN (
                                        SELECT CDFILIAL 
                                            FROM FILIOPER 
                                            WHERE CDOPERADOR = :CDOPERADOR
                                    ))
                        ) FLC, ORIGEMVENDAFOS O
                        WHERE DTENTRVENDA BETWEEN :DATAINI AND :DATAFIN 
                            AND TO_DATE(TO_CHAR(DTVENDA, 'HH24MI'), 'HH24MI') BETWEEN TO_DATE(:HRINI, 'HH24MI') AND TO_DATE(:HRFIN, 'HH24MI') 
                            AND IDSITUVENDA = 'O' 
                            AND V.CDLOJA    = FLC.CDLOJA 
                            AND V.CDFILIAL  = FLC.CDFILIAL 
                            AND V.CDCAIXA   = FLC.CDCAIXA
                            AND O.IDORIGEMVENDA = V.IDORIGEMVENDA
                ) V, (
                    SELECT :IDIMPHR AS IDIMPHR 
                        FROM DUAL
                ) D, ITEMVENDA IT, PRODUTO P 
                WHERE V.CDFILIAL     = IT.CDFILIAL 
                    AND V.CDCAIXA    = IT.CDCAIXA 
                    AND V.NRSEQVENDA = IT.NRSEQVENDA 
                    AND IT.CDPRODUTO = P.CDPRODUTO 
                GROUP BY V.CDFILIAL, V.NMFILIAL, V.CDLOJA, V.NMLOJA, 
                        DECODE(D.IDIMPHR, 'S', V.DTVENDA, TO_CHAR(V.DTENTRVENDA, 'DD/MM/YYYY HH24:MI')), 
                        V.DTENTRVENDA, V.IDMOD, P.CDARVPROD, P.NMPRODUTO 
            ORDER BY V.CDFILIAL, V.CDLOJA, V.NMLOJA, 
                    DECODE(D.IDIMPHR, 'S', V.DTVENDA, TO_CHAR(V.DTENTRVENDA, 'DD/MM/YYYY HH24:MI')), V.IDMOD, P.CDARVPROD
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($paramRelatorio);
        
        $result = $std->fetchAll();
        
        // aplicando mascara no produto
        for($i = 0; $i < count($result); $i++) {
            $codigoProduto = $this->mascaraPersonalizada($result[$i]['CDARVPROD']);
            
            $result[$i]['PRODUTO'] = $codigoProduto.' - '.$result[$i]['NMPRODUTO'];
        }
        
        return $result;
    }
    
    private function mascaraPersonalizada($input) {
        // Remove qualquer caractere que não seja letra ou número
        $input = preg_replace('/[^a-zA-Z0-9]/', '', $input);
    
        // Aplica a máscara
        $mascara = preg_replace('/([a-zA-Z0-9]{1})(\d{1})([a-zA-Z0-9]{1})(\d{2})(\d{3})(\d{2})/', '$1.$2$3.$4.$5.$6', $input);
    
        return $mascara;
    }
}