<?php
namespace Teknisa\MAN\Services;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities\Query;
use Teknisa\MAN\Utilities;

class PDVService {
    
    protected $connection;
    protected $entityManager;
    
    public function __construct(EntityManager $entityManager, Connection $connection) {
        $this->entityManager = $entityManager;
        $this->connection = $connection;
    }
    
    public function exportarCaixas($params) {
        
        $paramsQuery = array(
            ":CDFILIAL"  => $params['filialAtual'],
            ":CDCAIXA"   => $params['caixaAtual'],
            ":CDLOJA"    => $params['lojaAtual'],
            ":NRORG"     => $params['NRORG']
        );
        $rowExport = $this->connection->fetchAll(Utilities\Query::SELECT_CAIXA, $paramsQuery)[0];
        
        for($i = 0; $i < count($params['filiCaixas']); $i++) {
            $filialToExport = $params['filiais'][$i];
            $caixaToExport = $params['caixas'][$i];
            $lojaToExport = $params['lojas'][$i];
            $rowExport[":OLD_CDFILIAL"] = $filialToExport;
            $rowExport[":OLD_CDCAIXA"] = $caixaToExport;
            $rowExport[":OLD_CDLOJA"] = $lojaToExport;
            $rowExport[":NRORG"] = $params['NRORG'];
            
            $this->connection->executeQuery(Utilities\Query::EXPORT_PARATRIZACAO, $rowExport);
        }
    }
    
    public function SaveBackup($params) {
        $paramsQuery = array(
            "NRSEQIMPRLOJA1BK" => $params["NRSEQIMPRLOJA1BK"],
            "NRSEQIMPRLOJA2BK"=> $params["NRSEQIMPRLOJA2BK"],
            "NRSEQIMPRLOJA5BK"=> $params["NRSEQIMPRLOJA5BK"],
            "CDCAIXA"       => $params["CDCAIXA"],
            "CDFILIAL"  => $params["CDFILIAL"],
            "NRORG"  => $params["NRORG"],
            "CDLOJA"  => $params["CDLOJA"]
            
        );
        $this->entityManager->getConnection()->executeQuery(Utilities\Query::SaveBackups, $paramsQuery);
        
    }
    
    public function FindBackup($params) {
        $paramsQuery = array(
            "CDCAIXA"       => $params["CDCAIXA"],
            "CDFILIAL"  => $params["CDFILIAL"],
            "NRORG"  => $params["NRORG"],
            "CDLOJA"  => $params["CDLOJA"]
            
        );
        $this->connection->fetchAll(Utilities\Query::FindBackup, $paramsQuery);
        
    }
    
    public function paramsImportExport($CDFILIAL, $CDCAIXA, $NRORG, $data, $rowName, $query) {
        $params = array(
            ":CDFILIAL"  => $CDFILIAL,
            ":CDCAIXA"   => $CDCAIXA,
            ":NRORG"     => $NRORG
        );
        
        $rowExport = $this->entityManager->getConnection()->fetchAll(Query::GET_PARAMS_EXPORT_IMPORT, $params);
        $rowExport = count($rowExport) > 0 ? preg_replace('/__/', '_', $rowExport[0][$rowName]) : "";
        
        for ($i = 1 ; $i < count($data); $i++) {
            if($data[$i] != null) {
                $pos = strpos($rowExport, $data[$i]);
                
                if ($pos === false) {
                    $concatenar = $data[$i];
                    $rowExport .= $data[$i] . "_";
                } else {
                    $concatenar = "";
                }
            }
        }
        
        $conditions = array(
            ":".$rowName  => '_'.$rowExport,
            ":CDFILIAL"   => $CDFILIAL,
            ":CDCAIXA"    => $CDCAIXA,
            ":NRORG"      => $NRORG
        );
        
        // verificando se a string dos caixas ultrapassa 250 caracteres
        if(strlen($conditions[":".$rowName]) > 250) {
            return false;
        } else {
            $this->connection->executeQuery($query, $conditions);
            
            return true;
        }
    }    
    
    public function deleteParamsImportExport($params, $data, $rowName) {
        
        $rowExport = $this->entityManager->getConnection()->fetchAll(Utilities\Query::GET_PARAMS_EXPORT_IMPORT, $params);
        $rowExport = count($rowExport) > 0 ? $rowExport[0][$rowName] : "";
        
        for($i = 1 ; $i < count($data); $i++) {
           $rowExport = str_replace("_".$data[$i]['CDCAIXA'], "", $rowExport);
        }
        $texto = preg_replace('/__/','_',$rowExport);
        
        $params = array_merge($params, array(":".$rowName  => $texto));
        
        if($rowName == 'DSCAIXAVEN'){
            $this->connection->executeQuery(Utilities\Query::UPDATE_PARAMS_EXPORT, $params);    
        }else{
            $this->connection->executeQuery(Utilities\Query::UPDATE_PARAMS_IMPORT, $params);
        }
        
    }
    
    public function deleteModalidades($NRORG, $CDFILIAL, $CDCAIXA) {
        $params = array(
            "CDFILIAL"  => $CDFILIAL,
            "CDCAIXA"   => $CDCAIXA,
            "NRORG"     => $NRORG
        );
        $this->connection->executeQuery(Query::DELETE_VND_RELACAIXAMODALIDADE, $params);
    }

    public function resetCaixa($NRORG, $CDFILIAL, $CDCAIXA, $CDLOJA, $DTULTATU, $NRORGULTATU, $CDOPERULTATU) {
        $params = array(
            ":DTULTATU"     => $DTULTATU,
            ":NRORGULTATU"  => $NRORGULTATU,
            ":CDOPERULTATU" => $CDOPERULTATU,
            ":CDFILIAL"     => $CDFILIAL,
            ":CDCAIXA"      => $CDCAIXA,
            ":CDLOJA"       => $CDLOJA,
            ":NRORG"        => $NRORG
        ); 
        $this->connection->executeQuery(Query::UPDATE_VND_RELALMOXARIFADOESTRUT_RESET_CAIXA, $params);
        $this->connection->executeQuery(Query::UPDATE_VND_RELACAIXAMODALIDADE, $params);
        $this->connection->executeQuery(Query::UPDATE_VND_CONFIGINDIVIDUALPDV, $params);
        $this->connection->executeQuery(Query::UPDATE_CAIXA, $params);
    }

    public function getNextNumModalidade() {
        return $this->connection->fetchAll(Query::GET_MAXNRRELCAIXAMODALIDADE);
    }

    public function findCD($NRORG, $CDFILIAL, $CDCAIXA) {
        $params = array(
            "CDFILIAL"     => $CDFILIAL,
            "CDCAIXA"      => $CDCAIXA,
            "NRORG"        => $NRORG
        );
        $cdcaixa = $this->connection->fetchAll(Query::GET_CDCAIXA, $params);
        return !isset($cdcaixa[0]['CDCAIXA']) ;
    }
    
    public function findIDATIVO($NRORG, $CDFILIAL, $CDCAIXA, $CDLOJA) {
        $params = array(
            "CDFILIAL"     => $CDFILIAL,
            "CDCAIXA"      => $CDCAIXA,
            "CDLOJA"       => $CDLOJA,
            "NRORG"        => $NRORG
        );
        $cdcaixa = $this->connection->fetchAll(Query::GET_CDCAIXA, $params);
        if (count($cdcaixa) == 0) return false;
        else return $cdcaixa[0]['IDATIVO'];   
    }
    
    public function findCDCaixa($NRORG, $CDFILIAL, $CDCAIXA, $CDLOJA) {
        $params = array(
            "CDFILIAL"     => $CDFILIAL,
            "CDCAIXA"      => $CDCAIXA,
            "NRORG"        => $NRORG,
            "CDLOJA"       => $CDLOJA
        );
        $cdcaixa = $this->connection->fetchAll(Query::GET_CDCAIXA, $params);
        return isset($cdcaixa[0]['CDCAIXA']) ? false : true;
    }
    
    public function relacionaVendedor($NRORG, $CDFILIAL, $CDCAIXA, $CDVENDEDOR) {
        $params = array(
            "CDCAIXA"    => $CDCAIXA,
            "CDFILIAL"   => $CDFILIAL,
            "CDVENDEDOR" => $CDVENDEDOR,
            "NRORG"      => $NRORG
        );
        $this->connection->executeQuery(Utilities\Query::RELACIONA_VENDEDOR, $params);
    }
    
    public function verificaVendedor($NRORG, $CDVENDEDOR) {
        $params = array(
            "CDVENDEDOR" => $CDVENDEDOR,
            "NRORG"      => $NRORG
        );
        $resultQuery = $this->connection->fetchAll(Utilities\Query::VERIFICA_VENDEDOR, $params);
        return $resultQuery[0]['CDCAIXA'];
    }
    
    public function desassociaVendedor($NRORG, $CDVENDEDOR) {
        $params = array(
            "CDVENDEDOR" => $CDVENDEDOR,
            "NRORG"      => $NRORG
        );
        $this->connection->executeQuery(Utilities\Query::DESASSOCIA_VENDEDOR, $params);
    }
    
    public function checkTipoDeCaixa($row){
        $params = array(
            "CDFILIAL"     => $row['CDFILIAL'],
            "CDCAIXA"      => $row['CDCAIXA'],
            "NRORG"        => $row['NRORG'],
            "CDLOJA"       => $row['CDLOJA']
        );
        $total = $this->connection->fetchAll(Utilities\Query::CHECK_TIPO_CAIXA, $params);
        return $total[0]['TOTAL'];
    }
    
    public function checkDeliverCentraliza($row){
        $params = array(
            "CDFILIAL"     => $row['CDFILIAL'],
            "CDCAIXA"      => $row['CDCAIXA'],
            "CDLOJA"       => $row['CDLOJA']
        );
            return $this->connection->fetchAll(Utilities\Query::CHECK_DELIVER_CENTRAL, $params);

    }
    
    // Verifica se existe venda para a série informada
    public function adqVendaSerieNfce($row){
        $params = array(
            "CDFILIAL"          => $row['CDFILIAL'],
            "CDCAIXA"           => $row['CDCAIXA'],
            "CDSERIENFCE"       => $row['CDSERIECX']

        );
            $result =  $this->connection->fetchAll(Utilities\Query::ADQVENDASERIENFCE, $params);
            $total = count($result);
            return $total;
    }
    
    // Se os números iniciais da nota e número de série do caixa são diferentes
    public function adqVenda24hrs($row){
        $params = array(
            "CDFILIAL"          => $row['CDFILIAL'],
            "CDSERIENFCE"       => $row['CDSERIECX']
        );
            $result =  $this->connection->fetchAll(Utilities\Query::ADQVENDA24HRS, $params);
            $total = count($result);
            return $total;
    }

    //verifica se o número de série já está sendo utilizado em outro caixa
    public function adqNrSerieNFCe($row){
        $params = array(
            "CDFILIAL"      => $row['CDFILIAL'],
            "CDSERIECX"     => $row['CDSERIECX'],
            "CDCAIXA"       => $row['CDCAIXA']
        );
            $result = $this->connection->fetchAll(Utilities\Query::ADQNRSERIENFCE, $params);
            $total = count($result);
            return $total;
    }
    
    public function adqNrMaiorNFCe($row){
        $params = array(
            "CDFILIAL"          => $row['CDFILIAL'],
            "CDSERIENFCE"       => $row['CDSERIECX'],
            "NRNOTAFISCALCE"    => $row['NRINICIONFCE']
        );
        
            $result =  $this->connection->fetchAll(Utilities\Query::ADQNRMAIORNFCE, $params);
            $total = count($result);
            return $total;
    }
    
    // Não existe venda para a série informada e o número inicial da nota foi informado
    public function adqNrIniNFCe($row){
        $params = array(
            "CDFILIAL"     => $row['CDFILIAL'],
            "CDCAIXA"      => $row['CDCAIXA']
        );
            $total =  $this->connection->fetchAll(Utilities\Query::ADQNRINICIONFCE, $params);
            return $total[0];
    }
    
    public function adqAmbNFCe($row){
        $params = array(
            "CDFILIAL"     => $row['CDFILIAL']
        );
            $total =  $this->connection->fetchAll(Utilities\Query::ADQAMBNFCE, $params);
            return $total[0];
    }
    
    public function upNovoNFCe($row) {
        $params = array(
            "CDCONTADOR"    => $row['CDCONTADOR'],
            "NRSEQUENCIAL"  => $row['NRSEQUENCIAL']
        );
        $this->connection->executeQuery(Utilities\Query::acmUpdNovoCodigoNFCE, $params);
    }
        
    public function novoCodigoNFCE($row){
        $params = array(
            "CDCONTADOR"    => $row['CDCONTADOR']
        );
            $total =  $this->connection->fetchAll(Utilities\Query::adqNovoCodigoNFCe, $params);
            return $total[0]['TOT'];
    }
    
    public function inserirNovoNFCe($row){
        $params = array(
            "CDCONTADOR"    => $row['CDCONTADOR'],
            "NRSEQUENCIAL"  => $row['NRSEQUENCIAL']
        );
        $this->connection->executeQuery(Utilities\Query::acmInsNovoCodigoNFCE, $params);
    }
    
    public function adqinscJur($row){
        $params = array(
            "CDFILIAL"     => $row['CDFILIAL']
        );
            $total =  $this->connection->fetchAll(Utilities\Query::INSCJUR, $params);
            return $total[0]['NRINSJURFILI'];
    }
    
    public function getCDSERIECX($row) {
        $params = [
            "CDFILIAL" => $row["CDFILIAL"],
            "CDCAIXA"  => $row["CDCAIXA"]
        ];
        
        $cdseriecx = $this->connection->fetchAll(Utilities\Query::getCDSERIECX, $params);
        
        return $cdseriecx[0]['CDSERIECX'];
    }

    public function findImpCupKDS($row){
        $params = array(
            "CDFILIAL"     => $row['CDFILIAL'],
            "CDCAIXA"      => $row['CDCAIXA'],
            "CDLOJA"       => $row['CDLOJA']
        );
        $caixa = $this->connection->fetchAll(Utilities\Query::GET_CAIXAIMPEXP, $params);

        if($caixa){
            return true;
        }else{
            return false;
        }
    }
    
    public function checkNumberSeries($row){
        $params = array(
            "CDFILIAL"  =>  $row['CDFILIAL'],
            "CDSERIECX" =>  $row['CDSERIECX'],
            "CDCAIXA"   =>  $row['CDCAIXA']
        );
        
        // Se essa consulta trouxer resultados há um erro na parametrização
        $resultado = $this->connection->fetchAll(Utilities\Query::CHECK_NUMBER_SERIES, $params);
    }
    
    public function adqNovoCodigoNfce($row){
        $params = array(
            'CDCONTADOR'  =>  $row['CDCONTADOR']
        );
        $result = $this->connection->fetchAll(Utilities\Query::ADQNOVOCODIGONFC, $params);
        $total = count($result);
        return $total;
    }
    
    public function updateNovoCodigoNfce($row){
        $this->connection->executeQuery(Utilities\Query::UPDATENOVOCODIGONFCE, $row);
    }
    
    public function insertNovoCodigoNfce($row){
        $this->connection->executeQuery(Utilities\Query::INSERTNOVOCODIGONFCE, $row);
    }
} 